"""Free Google Translate API for Python. Translates totally free of charge."""
__all__ = 'mkTranslator',
__version__ = '1.0.0'


from mkTranslation.client import mkTranslator
from mkTranslation.constants import LANGCODES, LANGUAGES